-- Test amenity_feature
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(46);

set search_path to camper, public;

select has_table('amenity_feature');
select has_pk('amenity_feature');
select table_privs_are('amenity_feature', 'guest', array['SELECT']);
select table_privs_are('amenity_feature', 'employee', array['SELECT']);
select table_privs_are('amenity_feature', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('amenity_feature', 'authenticator', array[]::text[]);

select has_column('amenity_feature', 'amenity_feature_id');
select col_is_pk('amenity_feature', 'amenity_feature_id');
select col_type_is('amenity_feature', 'amenity_feature_id', 'integer');
select col_not_null('amenity_feature', 'amenity_feature_id');
select col_hasnt_default('amenity_feature', 'amenity_feature_id');

select has_column('amenity_feature', 'amenity_id');
select col_is_fk('amenity_feature', 'amenity_id');
select fk_ok('amenity_feature', 'amenity_id', 'amenity', 'amenity_id');
select col_type_is('amenity_feature', 'amenity_id', 'integer');
select col_not_null('amenity_feature', 'amenity_id');
select col_hasnt_default('amenity_feature', 'amenity_id');

select has_column('amenity_feature', 'icon_name');
select col_is_fk('amenity_feature', 'icon_name');
select fk_ok('amenity_feature', 'icon_name', 'icon', 'icon_name');
select col_type_is('amenity_feature', 'icon_name', 'text');
select col_not_null('amenity_feature', 'icon_name');
select col_hasnt_default('amenity_feature', 'icon_name');

select has_column('amenity_feature', 'name');
select col_type_is('amenity_feature', 'name', 'text');
select col_not_null('amenity_feature', 'name');
select col_hasnt_default('amenity_feature', 'name');

select has_column('amenity_feature', 'position');
select col_type_is('amenity_feature', 'position', 'integer');
select col_not_null('amenity_feature', 'position');
select col_has_default('amenity_feature', 'position');
select col_default_is('amenity_feature', 'position', '2147483647');


set client_min_messages to warning;
truncate amenity_feature cascade;
truncate amenity cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into amenity (amenity_id, company_id, label, name)
values (26, 2, 'W1', 'Amenity W1')
     , (28, 4, 'B1', 'Amenity B1')
;

insert into amenity_feature (amenity_id, icon_name, name)
values (26, 'information', 'Feature 26.1')
     , (28, 'wifi', 'Feature 28.1')
;

prepare amenity_feature_data as
select amenity_id, name
from amenity_feature
;

set role guest;
select bag_eq(
	'amenity_feature_data',
	$$ values (26, 'Feature 26.1')
	        , (28, 'Feature 28.1')
	$$,
	'Everyone should be able to list all amenity features across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ insert into amenity_feature(amenity_id, icon_name, name) values (26, 'castle', 'Feature 26.2') $$,
	'Admin from company 2 should be able to insert a new amenity feature to that company.'
);

select bag_eq(
	'amenity_feature_data',
	$$ values (26, 'Feature 26.1')
	        , (26, 'Feature 26.2')
	        , (28, 'Feature 28.1')
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update amenity_feature set name = 'Feature 26-2' where amenity_id = 26 and name = 'Feature 26.2' $$,
	'Admin from company 2 should be able to update amenity feature of that company.'
);

select bag_eq(
	'amenity_feature_data',
	$$ values (26, 'Feature 26.1')
	        , (26, 'Feature 26-2')
	        , (28, 'Feature 28.1')
	$$,
	'The row should have been updated.'
);

select lives_ok(
	$$ delete from amenity_feature where amenity_id = 26 and name = 'Feature 26-2' $$,
	'Admin from company 2 should be able to delete amenity feature from that company.'
);

select bag_eq(
	'amenity_feature_data',
	$$ values (26, 'Feature 26.1')
	        , (28, 'Feature 28.1')
	$$,
	'The row should have been deleted.'
);

select throws_ok(
	$$ insert into amenity_feature (amenity_id, icon_name, name) values (28, 'toilet', 'Feature 28.2') $$,
	'42501', 'new row violates row-level security policy for table "amenity_feature"',
	'Admin from company 2 should NOT be able to insert new amenity features to company 4.'
);

select lives_ok(
	$$ update amenity_feature set name = 'Feature 28-1' where amenity_id = 28 $$,
	'Admin from company 2 should not be able to update amenitys of company 4, but no error if amenity_id is not changed.'
);

select bag_eq(
	'amenity_feature_data',
	$$ values (26, 'Feature 26.1')
	        , (28, 'Feature 28.1')
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update amenity_feature set amenity_id = 28 where amenity_id = 26 $$,
	'42501', 'new row violates row-level security policy for table "amenity_feature"',
	'Admin from company 2 should NOT be able to move amenity feature to one of company 4'
);

select lives_ok(
	$$ delete from amenity_feature where amenity_id = 28 $$,
	'Admin from company 2 should NOT be able to delete amenity from company 4, but not error is thrown'
);

select bag_eq(
	'amenity_feature_data',
	$$ values (26, 'Feature 26.1')
	        , (28, 'Feature 28.1')
	$$,
	'No row should have been changed'
);

select throws_ok(
	$$ insert into amenity_feature (amenity_id, icon_name, name) values (26, 'baby', '  ') $$,
	'23514', 'new row for relation "amenity_feature" violates check constraint "name_not_empty"',
	'Should not be able to insert amenity features with a blank name.'
);

reset role;


select *
from finish();

rollback;

